import { liftF } from './fn.js'
import { Free } from './free.js'
import { Algebra, Foldable, Functor, Monad } from './index.js'
import { none as None } from './option.js'

/** @template T */
export class List extends Algebra(Functor, Monad, Foldable) {
  /** @type {Free<T> | None<T>} */
  _head

  /** @type {List<T>} */
  _tail

  /**
   * @param {T | None} [head]
   * @param {List<T>} [tail=List]
   */
  constructor(head = None, tail = List.of(None)) {
    super()
    this._head = head
    this._tail = tail
  }

  /**
   * @template T
   * @param {T} value 
   * @returns {List<T>}
   */
  prepend(value) {
    return new List(value, this)
  }

  /**
   * @template T
   * @param {T} value 
   * @returns {List<T>}
   */
  static of(value) {
    return new List(value)
  }

  /**
   * @template T
   * @param {Iterable<T>} iterable 
   * @returns {List<T>}
   */
  static from(iterable) {
    Array.from(iterable).reduceRight((list, value))
  }
}

const list = liftF(1)
console.log(list.map(_ => 2).map(_ => 3).chain(x => x).run())

